﻿using System;
using System.Linq;
using System.IO;
using System.Xml.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace WplToITunesXml
{
	public class Playlist
	{
		[DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		private extern static
		int GetShortPathName (
			string lpszLongPath,
			StringBuilder lpszShortPath,
			int cchBuffer);

		private static string ToShortPath (string longPath)
		{
			StringBuilder result = new StringBuilder(260);
			if (GetShortPathName(longPath, result, 260) != 0)
			{
				return result.ToString();
			}
			return longPath;
		}

		private readonly string _name;
		private readonly Uri _baseUri;
		private readonly string[] _tracks;

		public Playlist (FileInfo file)
		{
			_baseUri = new Uri(file.FullName);
			var document = XDocument.Load(file.FullName);
			_name = (from elemSmil in document.Elements("smil")
						from elemHead in elemSmil.Elements("head")
						from elemTitle in elemHead.Elements("title")
						where !String.IsNullOrEmpty(elemTitle.Value)
						select elemTitle.Value).First();
			if (String.IsNullOrEmpty(_name))
			{
				_name = Path.GetFileNameWithoutExtension(file.Name);
			}

			_tracks = (from elemSmil in document.Elements("smil")
					  from elemBody in elemSmil.Elements("body")
					  from elemSeq in elemBody.Elements("seq")
					  from elemMedia in elemSeq.Elements("media")
					  from attrSrc in elemMedia.Attributes(XName.Get("src"))
					  select new Uri(ToShortPath(new Uri(_baseUri, attrSrc.Value).LocalPath)).AbsoluteUri).ToArray();
		}

		public string Name
		{
			get
			{
				return _name;
			}
		}

		public string[] Tracks
		{
			get
			{
				return _tracks;
			}
		}
	}
}
