﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;

namespace WplToITunesXml
{
	class Program
	{
		private class FileInfoEqualityComparer : IEqualityComparer<FileInfo>
		{
			public static readonly FileInfoEqualityComparer Instance = new FileInfoEqualityComparer();

			public bool Equals (FileInfo x, FileInfo y)
			{
				if (Object.ReferenceEquals(x, y))
				{
					return true;
				}
				if (Object.ReferenceEquals(x, null) || Object.ReferenceEquals(y, null))
				{
					return false;
				}
				return String.Compare(x.FullName, y.FullName, true) == 0;
			}

			public int GetHashCode (FileInfo obj)
			{
				return obj.FullName.ToUpper().GetHashCode();
			}
		}

		static void Main (string[] args)
		{
			if (args.Length < 2)
			{
				return;
			}

			FileInfo targetFile = new FileInfo(args[0]);
			string[] fileSetArgs = new List<string>(args).GetRange(1, args.Length - 1).ToArray();

			FileInfo[] files =
				(from file in
					 (from fileset in fileSetArgs
					  from file in Directory.GetFiles(Path.GetDirectoryName(fileset), Path.GetFileName(fileset))
					  select file).Distinct()
				 select new FileInfo(file)).ToArray();

			Playlist[] playlists = (from file in files
									select new Playlist(file)).ToArray();

			Array.ForEach(playlists, playlist =>
			{
				Console.WriteLine(playlist.Name);
			});

			string[] tracks = (from playlist in playlists
							   from file in playlist.Tracks
							   select file).Distinct().OrderBy(name => name).ToArray();

			using (XmlWriter writer = XmlWriter.Create(
				targetFile.FullName, new XmlWriterSettings
				{
					Indent = true,
					IndentChars = "\t"
				}))
			{
				writer.WriteStartDocument();
				writer.WriteDocType(
					"plist", "-//Apple Computer//DTD PLIST 1.0//EN", "http://www.apple.com/DTDs/PropertyList-1.0.dtd", null);
				writer.WriteStartElement("plist");
				writer.WriteStartAttribute("version");
				writer.WriteString("1.0");
				writer.WriteEndAttribute();
				writer.WriteStartElement("dict");

				writer.WriteStartElement("key");
				writer.WriteString("Tracks");
				writer.WriteEndElement();
				writer.WriteStartElement("dict");
				for (int i = 0; i < tracks.Length; ++i)
				{
					string track = tracks[i];
					writer.WriteStartElement("key");
					writer.WriteString((i + 1).ToString());
					writer.WriteEndElement();
					writer.WriteStartElement("dict");
					writer.WriteStartElement("key");
					writer.WriteString("Track ID");
					writer.WriteEndElement();
					writer.WriteStartElement("integer");
					writer.WriteString((i + 1).ToString());
					writer.WriteEndElement();
					writer.WriteStartElement("key");
					writer.WriteString("Track Type");
					writer.WriteEndElement();
					writer.WriteStartElement("string");
					writer.WriteString("File");
					writer.WriteEndElement();
					writer.WriteStartElement("key");
					writer.WriteString("Location");
					writer.WriteEndElement();
					writer.WriteStartElement("string");
					writer.WriteString(track);
					writer.WriteEndElement();
					writer.WriteEndElement();
				}
				writer.WriteEndElement();

				writer.WriteStartElement("key");
				writer.WriteString("Playlists");
				writer.WriteEndElement();
				writer.WriteStartElement("array");
				foreach (Playlist playlist in playlists)
				{
					writer.WriteStartElement("dict");
					writer.WriteStartElement("key");
					writer.WriteString("Name");
					writer.WriteEndElement();
					writer.WriteStartElement("string");
					writer.WriteString(playlist.Name);
					writer.WriteEndElement();
					writer.WriteStartElement("key");
					writer.WriteString("Playlist Items");
					writer.WriteEndElement();
					writer.WriteStartElement("array");
					foreach (string track in playlist.Tracks)
					{
						writer.WriteStartElement("dict");
						writer.WriteStartElement("key");
						writer.WriteString("Track ID");
						writer.WriteEndElement();
						writer.WriteStartElement("integer");
						writer.WriteString((Array.IndexOf(tracks, track) + 1).ToString());
						writer.WriteEndElement();
						writer.WriteEndElement();
					}

					writer.WriteEndElement();
					writer.WriteEndElement();
				}
				writer.WriteEndElement();

				writer.WriteEndElement();
				writer.WriteEndElement();
			}

		}
	}
}

